#!/usr/bin/python

lic = \
"""
Copywrite 2015 Waltraud Erber
This program is free software: 
you can redistribute it and/or modify
it under the terms of the GNU General i
Public License as published by
the Free Software Foundation, 
either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope 
that it will be useful, but WITHOUT ANY WARRANTY; 
without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
See the GNU General Public License for more details.

You should have received a copy of the 
GNU General Public License
along with this program.  
If not, see <http://www.gnu.org/licenses/>.
"""

import wx
import wx.lib.mixins.listctrl as listmix
import os
import sys
import os.path
import subprocess
import csv
import re
from collections import defaultdict

#--------------------------------------------
# Main form for exporting multiterm termbases
#--------------------------------------------
class ExportForm(wx.Panel):
	def __init__(self, parent, *args, **kwargs):
		wx.Panel.__init__(self, parent, *args, **kwargs)

		self.parent = parent
		initial_tablelist = []
		
		# Field definitions for export-form
		#----------------------------------
		
		# empty line
		self.filler = wx.StaticText(self, -1, '')
		# logging window at bottom of form
		self.logger = wx.TextCtrl(self, style=wx.TE_MULTILINE|wx.TE_READONLY)
		# multiterm file selection
		self.mtLabel = wx.StaticText(self, label="MT-File:")
		self.mtTextCtrl = wx.TextCtrl(self, value="", style=wx.TE_READONLY)
		# name of output file with extracted tables
		self.glLabel = wx.StaticText(self, \
			label="File for export (change default as necessary):")
		self.glTextCtrl = wx.TextCtrl(self, value="")
		# optional prefix for temporary table data
		self.prefLabel = wx.StaticText(self, \
			label="Prefix for filenames of exported tables (optional):")
		self.prefTextCtrl = wx.TextCtrl(self, value="")
		# optional separator for multiple terms in one language
		self.sepLabel = wx.StaticText(self, \
			label="Separator for multiple terms (default: //):")
		self.sepTextCtrl = wx.TextCtrl(self, value="")
		# list of language tables found in multiterm termbase
		self.tableListLabel = wx.StaticText(self, \
			label="Tables found:")
		self.tableList = wx.ListBox(self, 10, wx.DefaultPosition, (170,130), \
			initial_tablelist, wx.LB_MULTIPLE)
		# action buttons
		self.findTablesButton = wx.Button(self, label="Find Tables")
		self.exportTablesButton = wx.Button(self, label="Export Tables")
		self.clearButton = wx.Button(self, label="Clear Fields")

		# Layout of form using sizers
		#----------------------------
		vbox = wx.BoxSizer(wx.VERTICAL)
		vbox2 = wx.BoxSizer(wx.HORIZONTAL)
		vbox3 = wx.BoxSizer(wx.HORIZONTAL)
		grid1 = wx.FlexGridSizer(rows=2, cols=2, vgap=10, hgap=10)
		grid2 = wx.FlexGridSizer(rows=2, cols=2, vgap=10, hgap=10)
		vbox.Add(self.mtLabel, 0)
		vbox.Add(self.mtTextCtrl, flag=wx.EXPAND)
		vbox.Add(self.glLabel, 0)
		vbox.Add(self.glTextCtrl, flag=wx.EXPAND)
		grid2.Add(self.prefLabel, 0)
		grid2.Add(self.prefTextCtrl, flag=wx.EXPAND)
		grid2.Add(self.sepLabel, 0)
		grid2.Add(self.sepTextCtrl, flag=wx.EXPAND)
		vbox.Add(grid2, 0)
		vbox.Add(self.tableListLabel, 0)
		vbox2.Add(self.tableList,0)
		grid1.Add(self.findTablesButton, 0)
		grid1.Add(self.exportTablesButton, 0)
		grid1.Add(self.clearButton, 0)
		vbox2.Add(grid1, 0)
		vbox.Add(vbox2, 0)
		vbox.Add(self.filler, 0, flag=wx.EXPAND)
		vbox.Add(self.logger, proportion=1, flag=wx.EXPAND)
		self.SetSizer(vbox)

		# connecting action buttons to the actions
		#-----------------------------------------
		self.Bind(wx.EVT_BUTTON, self.findTables, self.findTablesButton)
		self.Bind(wx.EVT_BUTTON, self.exportTables, self.exportTablesButton)
		self.Bind(wx.EVT_BUTTON, self.clearFields, self.clearButton)


	# use command mdb-tables to list language tables in termbase
	#-----------------------------------------------------------
	def findTables(self, event):
		self.languages = []
		mtfile = self.mtTextCtrl.GetValue()
		tablePrefix = self.prefTextCtrl.GetValue()

		# check, if input termbase exists
		if not os.path.isfile(mtfile):
			self.message("Multiterm file does not exist")
			return

		# check, if Prefix is within allowed limits
		if tablePrefix != "" and not \
		re.compile("^[a-zA-Z0-9]{1,2}$").match(tablePrefix):
			self.message("Prefix must be max. 2 letter or numbers")
			return

		# execute mdb-tables command
		command = 'mdb-tables -1 '+mtfile
		p = subprocess.Popen(command, shell=True, stdout=subprocess.PIPE, \
			stderr=subprocess.STDOUT)
		for line in p.stdout:
			if line[:2] == "I_":
				self.languages.append(line.strip())
		retval = p.wait()

		# fill in list-box with language tables found
		self.tableList.Set(self.languages)

	#
	# export language tables first to intermediate csv-Files
	# and then from the intermediate files into a combined csv-file
	# containing the selected languages
	#---------------------------------------------------------------
	def exportTables(self, event):
		tables = []
		explang = []
		colheads = []
		files = []
		# create dictionary of lists
		g = defaultdict(list)
		# get data from form fields
		mtfile = self.mtTextCtrl.GetValue()
		prefix = self.prefTextCtrl.GetValue()
		tables = self.tableList.GetSelections()
		glossfile = self.glTextCtrl.GetValue()
		separator = self.sepTextCtrl.GetValue()

		# check, if separator is within allowed limits
		if separator != "" and not \
		re.compile("^[\s\w\/:;_#]{1,2}$").match(separator):
			self.message("Separator contains illegal characters or is too long")
			return
		if separator == "":
			separator = "//"
		
		for i in tables:
			explang.append(self.languages[i])

		if len(explang) < 2:
			self.message("Select at least two tables")
			return

		self.__log("Exporting language tables:")

		# for each selected language table, read terms in table and 
		# store in dictionary by conceptid
		for i in range(len(explang)):
			# cut of language name by first space
			sp = explang[i].find(' ')
			if not sp == -1:
				part1 = explang[i][:sp]
			else:
				part1 = explang[i]
			colheads.append(part1)
			if prefix == "":
				file = part1+".csv"
			else:
				file = prefix+'_'+part1+".csv"
			files.append(file)
			
			# open intermediate file
			# and write output of mdb-export command to it
			with open(file,"w") as f:
				self.__log(explang[i])
				command = 'mdb-export -b octal '+mtfile+' "'+explang[i]+'"'
				p = subprocess.Popen(command, shell=True, \
					stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
				for line in p.stdout.readlines():
					f.write(line)
				retval = p.wait()

		# read all intermediate files and combine terms first in
		# dictionary then in output file
		fileCount = 0
		for file in files:
			fileCount = fileCount+1
			message = "Reading "+file
			self.__log(message)
			with open(file, "r") as f:
				csvContent = csv.reader(f)
				for row in csvContent:
					if row[0] != "sortterm":
						key = row[3]
						if len(g[key]) == fileCount:
							g[key][fileCount-1] = g[key][fileCount-1]+ \
								' '+str(separator)+' '+row[1]
						elif len(g[key]) < fileCount-1:
							i = fileCount -1 - len(g[key])
							while i > 0:
								g[key].append("")
								i = i-1
								g[key].append(row[1])
						else:
							g[key].append(row[1])
		
		glossdir = os.path.dirname(glossfile)
		if not os.path.exists(glossdir):
			self.message("Path to output file does not exist")
			return

		if os.path.isfile(glossfile):
			message = glossfile+" already exists. Overwrite?"
			if self.message2(message) == wx.ID_OK:
				self.WriteOutputFile(glossfile, colheads, g)
		else:
			self.WriteOutputFile(glossfile, colheads, g)

	# Clear all fields 
	#-----------------
	def clearFields(self, event):
		emptylist = []
		self.mtTextCtrl.SetValue("")
		self.prefTextCtrl.SetValue("")
		self.tableList.Set(emptylist)
		self.glTextCtrl.SetValue("")
		self.sepTextCtrl.SetValue("")


	# Message popup
	#--------------
	def message(self, textMessage):
		dlg = wx.MessageDialog(self, textMessage, "", \
			wx.OK|wx.ICON_INFORMATION|wx.TE_MULTILINE)
		dlg.ShowModal()
		dlg.Destroy()


	# OK/Cancel popup
	#--------------
	def message2(self, textMessage):
		dlg = wx.MessageDialog(self, textMessage, "", \
			wx.OK|wx.CANCEL|wx.ICON_INFORMATION)
		result = dlg.ShowModal()
		dlg.Destroy()
		return result


	# Output to text-control field
	#------------------------------
	def __log(self, message):
		''' Private method to append a string to the logger text
			control. '''
		self.logger.AppendText('%s\n'%message)


	# Write Output
	#-------------
	def WriteOutputFile(self, glossary, colheads, data):
		with open(glossary, "w") as f:
			message = "Writing output file: "+glossary
			self.__log(message)
			for i in range(len(colheads)):
				string = colheads[i]+','
				f.write(string),
			f.write("\n")
			for key in data:
				for i in range(len(data[key])):
					string = '"'+data[key][i]+'"'+','
					f.write(string),
				f.write("\n")


#----------------------
# Outer frame
#----------------------
class OuterFrame(wx.Frame):
	def __init__(self, parent, id, title):
		wx.Frame.__init__(self, parent, id, title, wx.DefaultPosition, \
			wx.Size(500,550))
		initial_tablelist = []

		
		# create and fill main menu bar
		#------------------------------
		menubar = wx.MenuBar()
		file = wx.Menu()
		help = wx.Menu()
		file.Append(101, '&Select Input', 'Select MT-File')
		file.AppendSeparator()
		quit = wx.MenuItem(file, 105, '&Quit\tCtrl+Q', 'Quit')
		file.AppendItem(quit)
		m_about = help.Append(wx.ID_ABOUT, "&About", "Info about mt-export")
		m_lic = help.Append(wx.ID_ANY, "&License", "Licensed under GPL")
		menubar.Append(file, '&File')
		menubar.Append(help, '&Help')
		self.SetMenuBar(menubar)
		self.Bind(wx.EVT_MENU, self.OnQuit, quit)
		self.Bind(wx.EVT_MENU, self.selectfile, id=101)
		self.Bind(wx.EVT_MENU, self.OnAbout, m_about)
		self.Bind(wx.EVT_MENU, self.OnLic, m_lic)

		self.Panel = ExportForm(self, size=(500,500))
		self.Fit()

	def selectfile(self, event):
		dlg = wx.FileDialog(self, "Choose MT-File", os.getcwd(), "", "*.*", \
			wx.OPEN)
		if dlg.ShowModal() == wx.ID_OK:
			path = dlg.GetPath()
			self.Panel.mtTextCtrl.SetValue(path)
			defaultExport = os.path.dirname(path)+"/gl-export.csv"
			self.Panel.glTextCtrl.SetValue(defaultExport)
		dlg.Destroy()

	def OnQuit(self, event):
		self.Close()

	def OnAbout(self, event):
		info = """ 
mt-export exports language tables contained in 
multiterm termbase to csv-files.

Prerequisites: mdbtools must be installed
Steps: 
 - Use File->Select to select multiterm termbase
 - Modify output file and add prefix for temporary 
   files as needed
 - Change the default separator for multiple terms 
   if needed
 - Use 'Find Tables' to get a list of language tables
 - Use 'Export Tables' to export tables to csv-glossary """

		self.Panel.message(info)
		
	def OnLic(self, event):
		self.Panel.message(lic)


class MyApp(wx.App):
	def OnInit(self):
		frame = OuterFrame(None, -1, 'mt-export')	
		frame.Show(True)
		return True

app = MyApp(0)
app.MainLoop()
